from .api import Api
from .entities import (
    BaseEntity,
    Client,
    Dashboard,
    Group,
    Project,
    ProjectUser,
    Tag,
    Task,
    TimeEntry,
    User,
    Workspace,
    WorkspaceUser,
)


class BaseRepository(Api):
    LIST_URL = ""
    DETAIL_URL = LIST_URL + "/{id}"
    ENTITY_CLASS = BaseEntity

    def retrieve(self, id: int = None, **kwargs):
        full_url = self.BASE_URL.join(self.DETAIL_URL.format(id=id))
        response = self.get(full_url)
        return self.ENTITY_CLASS(**response.json())

    def list(self, **kwargs):
        full_url = self.BASE_URL.join(self.LIST_URL)
        response = self.get(full_url)
        return [self.ENTITY_CLASS(**entity) for entity in response.json()]

    def create(self, entity: ENTITY_CLASS):
        full_url = self.BASE_URL.join(self.LIST_URL)
        response = self.post(full_url, data=entity.dict())
        return self.ENTITY_CLASS(**response.json())

    def update(self, entity: ENTITY_CLASS):
        full_url = self.BASE_URL.join(self.DETAIL_URL.format(id=entity.id))
        response = self.put(full_url, data=entity.dict())
        return self.ENTITY_CLASS(**response.json())

    def partial_update(self, entity: ENTITY_CLASS):
        full_url = self.BASE_URL.join(self.DETAIL_URL.format(id=entity.id))
        response = self.patch(full_url, data=entity.dict())
        return self.ENTITY_CLASS(**response.json())


class Clients(BaseRepository):
    LIST_URL = "clients"
    ENTITY_CLASS = Client


class Groups(BaseRepository):
    LIST_URL = "groups"
    ENTITY_CLASS = Group


class Projects(BaseRepository):
    LIST_URL = "projects"
    ENTITY_CLASS = Project


class ProjectUsers(BaseRepository):
    LIST_URL = "project_users"
    ENTITY_CLASS = ProjectUser


class Tags(BaseRepository):
    LIST_URL = "tags"
    ENTITY_CLASS = Tag


class Tasks(BaseRepository):
    LIST_URL = "tasks"
    ENTITY_CLASS = Task


class TimeEntries(BaseRepository):
    LIST_URL = "time_entries"
    ENTITY_CLASS = TimeEntry


class Users(BaseRepository):
    LIST_URL = "users"
    ENTITY_CLASS = User


class Workspaces(BaseRepository):
    LIST_URL = "workspaces"
    ENTITY_CLASS = Workspace


class WorkspaceUsers(BaseRepository):
    LIST_URL = "workspace_users"
    ENTITY_CLASS = WorkspaceUser


class Dashboards(BaseRepository):
    LIST_URL = "dashboards"
    ENTITY_CLASS = Dashboard



import typing

import httpx


class BasicAuth(httpx.BasicAuth):
    pass


class TokenAuth(httpx.BasicAuth):
    SECRET: str = "api_token"

    def __init__(self, token: typing.Union[str, bytes]):
        self.auth_header = self.build_auth_header(token, self.SECRET)



from datetime import datetime
from typing import Callable, List, Optional, Union

from pydantic import BaseModel, EmailStr, HttpUrl


class BaseEntity(BaseModel):
    id: Optional[int] = None
    at: Optional[datetime] = None


class Client(BaseEntity):
    name: str
    wid: int
    notes: Optional[str] = None


class Group(BaseEntity):
    name: str
    wid: int


class Project(BaseEntity):
    name: str
    wid: int
    cid: Optional[int] = None
    active: bool = True
    is_private: bool = True
    template: Optional[bool] = None
    template_id: Optional[int] = None
    billable: bool = True
    auto_estimates: Optional[bool] = False
    estimated_hours: Optional[int] = None
    color: Union[str, int] = None
    rate: Optional[float] = None
    created_at: Optional[datetime] = None


class ProjectUser(BaseEntity):
    pid: int
    uid: int
    wid: int
    notes: Optional[str] = None
    manage: Optional[bool] = False
    rate: Optional[float] = None
    full_name: Optional[float] = None


class Tag(BaseEntity):
    name: str
    wid: int


class Task(BaseEntity):
    name: str
    pid: int
    wid: int
    uid: Optional[int] = None
    estimated_seconds: Optional[int] = None
    active: Optional[bool] = True


class TimeEntry(BaseEntity):
    wid: int
    pid: Optional[int] = None
    tid: Optional[int] = None
    description: Optional[str] = None
    billable: Optional[bool] = False
    start: Union[datetime, Callable] = datetime.now
    stop: Union[datetime, Callable] = None
    duration: int
    created_with: Optional[str]
    tags: List[str] = []
    duronly: Optional[bool] = None


class User(BaseEntity):
    api_token: Optional[str] = None
    default_wid: Optional[int] = None
    email: EmailStr
    fullname: str
    jquery_timeofday_format: str
    jquery_date_format: str
    timeofday_format: str
    date_format: str
    store_start_and_stop_time: bool
    beginning_of_week: int = 0
    language: str
    image_url: HttpUrl
    sidebar_piechart: bool
    new_blog_post: dict
    send_product_emails: bool
    send_weekly_report: bool
    send_timer_notifications: bool
    openid_enabled: bool
    timezone: str


class Workspace(BaseEntity):
    name: str
    premium: bool
    admin: bool
    default_hourly_rate: float
    default_currency: str
    only_admins_may_create_projects: bool
    only_admins_see_billable_rates: bool
    rounding: int
    rounding_minutes: int
    logo_url: HttpUrl


class WorkspaceUser(BaseEntity):
    uid: int
    admin: bool
    active: bool
    invite_url: HttpUrl


class Activity(BaseEntity):
    user_id: int
    project_id: int
    duration: int
    description: str
    stop: datetime
    tid: int


class MostActiveUser(BaseEntity):
    user_id: int
    duration: int


class Dashboard(BaseEntity):
    most_active_user: List[MostActiveUser] = []
    activity: List[Activity]



"""Module contains simple Web API wraper."""

import typing
from functools import partial

import httpx

from .auth import BasicAuth, TokenAuth
from .exceptions import STATUS_2_EXCEPTION


class Api:
    """Simple api wraper."""

    BASE_URL: httpx.URL = httpx.URL("https://www.toggl.com/api/v8/")
    HEADERS = {"content-type": "application/json"}

    def __init__(
        self,
        base_url: typing.Optional[str] = None,
        auth: typing.Union[BasicAuth, TokenAuth] = None,
    ):
        if base_url:
            self.BASE_URL = httpx.URL(base_url)
        self.client = httpx.Client(base_url=self.BASE_URL, auth=auth)

    def __getattr__(self, httpmethod):
        try:
            method = getattr(self.client, httpmethod)
        except AttributeError:
            raise AttributeError(f"No such http method ({httpmethod})!")

        return partial(self.api_method, method)

    def api_method(
        self,
        method: typing.Callable,
        url: str,
        params: dict = None,
        data: dict = None,
        files: dict = None,
    ) -> httpx.Response:
        """
        Call http method with specified url and params
        """

        _url = self.BASE_URL.join(url)

        response = (
            method(_url, headers=self.HEADERS)
            if method.__name__ == "get"
            else method(_url, json=data, files=files)
        )

        if response.status_code == httpx.codes.OK:
            return response
        else:
            exception_class = STATUS_2_EXCEPTION[response.status_code]
            raise exception_class



__version__ = "0.1.1"

from .auth import BasicAuth, TokenAuth  # noqa: F401
from .entities import Dashboard  # noqa: F401
from .entities import (
    Activity,
    BaseEntity,
    Client,
    Group,
    MostActiveUser,
    Project,
    ProjectUser,
    Tag,
    Task,
    TimeEntry,
    User,
    Workspace,
    WorkspaceUser,
)
from .repository import Projects  # noqa: F401
from .repository import (
    Clients,
    Dashboards,
    Groups,
    ProjectUsers,
    Tags,
    Tasks,
    TimeEntries,
    Users,
    Workspaces,
    WorkspaceUsers,
)

__all__ = [
    "Activity",
    "BasicAuth",
    "BaseEntity",
    "Client",
    "Clients",
    "Dashboard",
    "Dashboards",
    "Group",
    "Groups",
    "MostActiveUser",
    "Project",
    "Projects",
    "ProjectUser",
    "ProjectUsers",
    "Tag",
    "Tags",
    "Task",
    "Tasks",
    "TimeEntries",
    "TimeEntry",
    "TokenAuth",
    "User",
    "Users",
    "Workspace",
    "Workspaces",
    "WorkspaceUser",
    "WorkspaceUsers",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['toggl_python']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.13.3,<0.14.0', 'pydantic[email]>=1.5.1,<2.0.0']

setup_kwargs = {
    'name': 'toggl-python',
    'version': '0.1.1',
    'description': 'Python wraper for Toggl API.',
    'long_description': 'Toggl Python API\n================\n\n![https://pypi.python.org/pypi/toggl_python](https://img.shields.io/pypi/v/toggl_python.svg) ![https://travis-ci.com/evrone/toggl_python](https://img.shields.io/travis/evrone/toggl_python.svg) ![https://toggl-python.readthedocs.io/en/latest/?badge=latest](https://readthedocs.org/projects/toggl-python/badge/?version=latest) ![https://pyup.io/repos/github/evrone/toggl_python/](https://pyup.io/repos/github/evrone/toggl_python/shield.svg)\n\nToggl Python API\n----------------\n\n* Based on open Toggl API documentation: https://github.com/toggl/toggl_api_docs/blob/master/toggl_api.md\n* Free software: MIT license\n* Documentation: https://toggl-python.readthedocs.io.\n\nInstallation\n------------\n`pip install toggl-python` or use [poetry](https://python-poetry.org) `poetry add toggl-python`\n\nFeatures\n--------\n\n- Get TimeEntries.\n\n```python\nfrom toggl_python import TokenAuth, TimeEntries\n\nif __name__ == "__main__":\n    auth = TokenAuth(\'AUTH_TOKEN\')\n    print(TimeEntries(auth=auth).list())\n```\n\n* *TODO*\n\nCredits\n-------\n\nThis package was created with [Cookiecutter](https://github.com/audreyr/cookiecutter) and the [audreyr/cookiecutter-pypackage](https://github.com/audreyr/cookiecutter-pypackage) project template.\n',
    'author': 'Ivlev Denis',
    'author_email': 'me@dierz.pro',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/evrone/toggl_python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
